module(..., package.seeall)
-------------------------------------------------------------------------------
-- Authentication -------------------------------------------------------------
-------------------------------------------------------------------------------

PASSWORD_TEMPLATE = [=[USERS = {}
$do_users[[USERS["$user"]={hash="$hash", time="$time"}
]]]=]

function make_authenticator (sputnik)

   local password_page = sputnik:get_page(sputnik.config.PASS_PAGE_NAME)
   password_page = sputnik:decorate_page(password_page)
   local users = sputnik:load_config(password_page.content)
   users = users.USERS or {}

   local function get_token(username) 
      return md5.sumhexa(username..sputnik.config.SECRET_CODE.."Sputn!k") 
   end

   local function user_exists(username) 
      return users[username]
   end

   local function get_salted_hash(time, password)
      return md5.sumhexa(time..sputnik.config.SECRET_CODE..password)
   end
  
   local function now() 
      local d = os.date("*t")
      return string.format("%04d-%02d-%02dT%02d-%02d-%02d", d.year, d.month, d.day, d.hour, d.min, d.sec)
   end

   local function days_since_ce(date)
      return 365*tonumber(date:sub(0,4))+31*tonumber(date:sub(6,7))+tonumber(date:sub(9,10))
   end
   
   local function is_recent(username)
      if username and users[username] and users[username].time then
         return (days_since_ce(now()) -days_since_ce(users[username].time)) < 14
      else
         --sputnik.logger:debug(username.." not known, will return false")
         return false
      end
   end
    
   local function add_user(username, password) 
      local time = now()
      users[username] = {
         hash  = get_salted_hash(time, password), 
         time = time
      }
   end

   local function save(user)
      local new_values = {
         content = cosmo.f(PASSWORD_TEMPLATE){ 
                      do_users = function()
                         for user, value in pairs(users) do
                            cosmo.yield{
                               user=user, 
                               hash=value.hash,
                               time=value.time}
                         end
                      end
                   },
         author  = user,
         summary = "Added new user: "..user,
         minor   = "yes",
      }
      password_page = sputnik:update_page_with_params(password_page,
                                                      new_values)
      password_page = sputnik:decorate_page(password_page)
      
      sputnik:save_page(password_page)
   end

   local function check_password(user, password) 
      if users[user] and users[user].hash == get_salted_hash(users[user].time, password) then
         return user, get_token(user)
      elseif user_exists(user) then 
	      return nil
      else
	      add_user(user, password)
	      save(user)
	      return user, get_token(user)
      end
   end

   local function check_token(user, token)
      if ((user or ""):len() > 0) and (get_token(user) == token) then
	      return user
      else
	      return nil
      end
   end

   return { 
      check_password = check_password,
      check_token    = check_token,
      is_recent      = is_recent,
   }
end
